#include <system.h>

#define FLIGHT_DATA_RECORDER // to indicate to some includes that this is the FDR

#include "buspins.h"
#include "common/bus.h"
#include "flash.h"
#include "eeprom.h"

#pragma DATA _CONFIG, _INTRC_OSC_NOCLKOUT & _CP_OFF & _CPD_OFF & _PWRTE_ON & _WDT_OFF & _MCLRE_OFF & _BOD_ON & _FCMEN_ON
#pragma DATA _EEPROM, 0, 0
#pragma CLOCK_FREQ 8000000

// constants
#define SAMPLES_PER_PAGE	28
#define FLASH_PAGE_ADDR		0 /* address in EEPROM containing the current page in the external flash */

// globals
bool takeSample = false; // set to true by the ISR every 20ms (50Hz)
unsigned char msSinceLastSample = 0;
struct Record sample;
bit LED = 0;
unsigned char ext1 = 0;
unsigned short flashPage = 0;

// analog ports
#define LONG_ACCEL			0b00001000 // AN2
#define IGNITER_VOLTAGE		0b00001100 // AN3
#define VERT_ACCEL			0b00010000 // AN4
#define BATT111_VOLTAGE		0b00011100 // AN7
#define BATT074_VOLTAGE		0b00100000 // AN8
#define ALTITUDE			0b00101000 // AN10

// digital ports
#define MAS_STAGING_SIG		portc.5
#define MC2_STAGING_SIG		porta.3
#define MAS_SERVO_SIG		portc.4
#define MC2_SERVO_SIG		portb.7
#define BUTTON				portc.2

unsigned char flash_test();

inline void enableInterrupts()
{
	set_bit(intcon, GIE);
}

inline void disableInterrupts()
{
	clear_bit(intcon, GIE);
}

unsigned short readADC(unsigned char channel)
{
	adcon0 = 0x81 | channel;
	set_bit(adcon0, GO);
	
	while (test_bit(adcon0, NOT_DONE))
		;
		
	unsigned short result;
	
	asm
	{
		movf _adresl, W
		movwf _result
		movf _adresh, W
		movwf _result+1
	}
	
	return result;
}

unsigned char readADCmsb(unsigned char channel)
{
	adcon0 = 0x01 | channel;
	set_bit(adcon0, GO);
	
	while (test_bit(adcon0, NOT_DONE))
		;
	
	return adresh;
}

void main()
{
	// retrieve ocscal and set it
	/*asm
	{
		bsf _status, RP0  	; Bank 1
		call 0x7ff 	; Get the cal value
		movwf _osctune 	; Calibrate
		bcf _status, RP0 	; Bank 0
	}*/
	
	// set oscillator to 8mhz
	set_bit(osccon, IRCF0);
	set_bit(osccon, IRCF1);
	set_bit(osccon, IRCF2); 

	// set appropriate outputs
	trisb.5 = 0;
	trisb.6 = 0;
	trisc.1 = 0;
	trisc.7 = 0;
	trisb.5 = 0;
	trisa.5 = 0;

	BUS_SYNC = 0;
	BUS_CLOCK = 0;
	BUS_DATA = 0;
	FLASH_SELECT = 1;
	FLASH_CLOCK = 0;
	
	// disable all unnecessary A/D inputs
	ansel = 0x9C;
	ansel1 = 0x05;
	adcon1 = 0x50; // 1:16 Tosc
		
	// set up timer
	clear_bit(option_reg, T0CS); // TMR0 operates on internal clock
	clear_wdt();
	clear_bit(option_reg, PSA); // prescaler is assigned to TMR0
	
	// set prescaler to 010 (1:8, approx every 1ms)
	clear_bit(option_reg, PS2);
	set_bit(option_reg, PS1);
	clear_bit(option_reg, PS0);
	
	enableInterrupts();
	
	set_bit(intcon, T0IE); // enable TMR0 interrupt
	
	zeroRecord(&sample);
	
	unsigned char sampleCounter = 0;
	unsigned char buttonTimeout = 0;
	bool replayMode = false;
	bool deleteAfterReplay = false;
	unsigned short recordedPages = eeprom_readShort(FLASH_PAGE_ADDR);
	
	delay_ms(50);
	
	for (buttonTimeout = 0; BUTTON == 0; ++buttonTimeout)
	{
		if (buttonTimeout > 250)
		{
			replayMode = true;
			break;
		}
		
		delay_ms(10);
	}
	
	// REPLAY MODE ==================================================
	if (replayMode) 
	{
		buttonTimeout = 0;
		flashPage = 0;
		
		flash_prepPageForRead(flashPage);
		
		while (flashPage <= recordedPages)
		{
			flash_readRecord(&sample);
			
			// verify checksum
			if (calcChecksum(&sample) != sample.checksum)
				sample.digital |= NO_DATA;
			else
				sample.digital &= ~NO_DATA;
			
			sample.digital |= REPLAY;
			
			if (deleteAfterReplay)
				sample.digital |= DELETE;
			
			sendRecord(&sample, 0);
			
			++sampleCounter;
			
			if (sampleCounter >= SAMPLES_PER_PAGE)
			{
				flash_endPageRead();
				++flashPage;
				flash_prepPageForRead(flashPage);
				sampleCounter = 0;
			}
			
			delay_ms(50); // so we don't dump the data faster than the serial encoder can read it
			
			if (BUTTON == 0)
				++buttonTimeout;
				
			deleteAfterReplay = buttonTimeout > 100;
		}
		
		if (deleteAfterReplay)
		{
			flash_eraseAll();
			eeprom_writeShort(FLASH_PAGE_ADDR, 0);
			zeroRecord(&sample);
						
			sample.digital = NO_DATA | DELETE;
			
			while (1)
			{
				sendRecord(&sample, 0);
				delay_ms(50);
			}
		}
		
		while (1);	
	}
	
	// RECORD MODE ====================================
	
	flashPage = eeprom_readShort(FLASH_PAGE_ADDR);
	
	flash_prepPageForWrite(flashPage);
	
	while (1)
	{
		if (takeSample)
		{
			
			//if (counter >= 25)
			{
				LED ^= 1;
				//test_var = flash_test();
				//counter = 0;
			}
			// take digital samples
			sample.digital = 0;
			
			if (MAS_SERVO_SIG)
				sample.digital |= MAS_SERVO;
			
			if (!MC2_SERVO_SIG) // active low
				sample.digital |= MC2_SERVO;
				
			if (MAS_STAGING_SIG)
				sample.digital |= MAS_STAGING;

			if (!MC2_STAGING_SIG) // active low
				sample.digital |= MC2_STAGING;
				
			if (BUTTON)
				sample.digital |= DEBUG_LED;

			// take analog samples
			SetLongAccel(sample, readADC(LONG_ACCEL));
			SetVertAccel(sample, readADC(VERT_ACCEL));
			SetAltitude(sample, readADC(ALTITUDE));
			sample.batt074 = readADCmsb(BATT074_VOLTAGE);
			sample.batt111 = readADCmsb(BATT111_VOLTAGE);
			sample.igniter = readADCmsb(IGNITER_VOLTAGE);
			
			// send the sample
			sendRecord(&sample, 1);
			takeSample = false;
			
			++sampleCounter;
			if (sampleCounter >= SAMPLES_PER_PAGE)
			{
				flash_endPageWrite();
				++flashPage;
				eeprom_writeShort(FLASH_PAGE_ADDR, flashPage);
				flash_prepPageForWrite(flashPage);
				sampleCounter = 0;
			}
		}
	}
}

void interrupt( void )
{
	if (test_bit(intcon, T0IF))
	{
		tmr0 = 40; // give it an extra 40 (8*5) every interrupt so that it will be very close to 1ms
		clear_bit(intcon, T0IF);  //clear TMR0 overflow flag
		
		++msSinceLastSample;
		if (msSinceLastSample >= 20)
		{
			takeSample = true;
			msSinceLastSample = 0;
		}
	}
}
