#ifndef RECORD_H_
#define RECORD_H_

#ifdef __GNUC__
	#include <stdint.h>
	#pragma pack(0)

#else
	#ifdef _BOOSTC
		typedef unsigned char uint8_t;
		typedef unsigned short uint16_t;
	#endif
#endif

// digital masks
/*
explanation of the NO_DATA state:
	if it is set alone, it means the telemetry module isn't communicating with the FDR
	if is set with REPLAY, it means the checksum failed coming out of the flash memory
	if is set with DELETE, it means the memory has been erased and is ready to be restarted
	if it iset set with both REPLAY and DELETE, the checksum failed and it is going to erased as soon as all the records are replayed
*/

#define NO_DATA			0x01
#define REPLAY			0x02
#define	DELETE			0x04
#define MAS_STAGING		0x08
#define MAS_SERVO		0x10
#define MC2_STAGING		0x20
#define MC2_SERVO		0x40
#define DEBUG_LED		0x80	

// analog masks (for extracting the appropriate bits)
#define ALTITUDE_MASK	0x03FF
#define VERTACCEL_MASK	0x3000
#define LONGACCEL_MASK	0xC000

// access macros
#define GetAltitude(rec)		(rec.altitude & ALTITUDE_MASK)
#define GetLongAccel(rec)		(((unsigned short)rec.longAccel << 2) | ((rec.altitude >> 14) & (unsigned short)0x0003))
#define GetVertAccel(rec)		(((unsigned short)rec.vertAccel << 2) | ((rec.altitude >> 12) & (unsigned short)0x0003))

#define SetAltitude(rec, alt)	\
	rec.altitude ^= rec.altitude & ALTITUDE_MASK; \
	rec.altitude |= (unsigned short)(alt)

#define SetLongAccel(rec, acc)	\
	rec.longAccel = (acc) >> 2; \
	rec.altitude ^= rec.altitude & LONGACCEL_MASK; \
	rec.altitude |= (unsigned short)(acc) << 14

#define SetVertAccel(rec, acc)	\
	rec.vertAccel = (acc) >> 2; \
	rec.altitude ^= rec.altitude & VERTACCEL_MASK; \
	rec.altitude |= (unsigned short)((unsigned short)(acc) & (unsigned short)0x0003) << 12

struct Record
{
	uint8_t digital;
			
	// analog flight data
	uint8_t longAccel;	// longitudinal acceleration (low 2 bits in altitude)
	uint8_t vertAccel;	// vertical acceleration [when flying] (low 2 bits in altitude)
	uint16_t altitude;	// altitude (in low 10 bits)
	uint8_t batt074;	// voltage of 7.4V battery
	uint8_t batt111;	// voltage of 11.1V battery
	uint8_t igniter;	// voltage at igniter output
	
	uint8_t checksum;
}
#ifdef __GNUC__
__attribute__((__packed__))
#endif
;

// calculate the checksum for a record (but doesn't update the checksum field in the record)

#ifdef __cplusplus
 extern "C" {
 #endif

uint8_t calcChecksum(const struct Record * rec);
void zeroRecord(struct Record * rec);

 #ifdef __cplusplus
 }
 #endif

#endif /*RECORD_H_*/
