#include <system.h>
#include "flash.h"

extern bit LED;
extern unsigned char ext1;

/* NOTE: all commands are shifted in and out of the flash memory MSB first, but the user data is going to
be stored LSB first because that is the order the bus uses for communicating w/ other PICs */

// instructions for the flash mem
#define WRITE_ENABLE	0x06
#define PAGE_PROGRAM	0x02
#define SECTOR_ERASE	0xD8
#define BULK_ERASE		0xC7
#define READ			0x03

// drive chip select low and shift out an 8-bit instruction.  leaves chip select low.
void flash_beginInstruction(uint8_t instr)
{
	FLASH_SELECT = 0;
	flash_shiftOutMSB(instr);
}

// drive chip select high
void flash_endInstruction()
{
	FLASH_SELECT = 1;
}

// shift out 8 bits of data, MSB first.  leaves clock low.
void flash_shiftOutMSB(uint8_t val)
{
	unsigned char bitIndex;
	
	// send each bit of the byte (one clock period per iteration)
	for (bitIndex = 0; bitIndex < 8; ++bitIndex)
	{
		FLASH_CLOCK = 0;
		FLASH_DATA_OUT = val.7;
		val <<= 1;
		FLASH_CLOCK = 1;
	}
	
	FLASH_CLOCK = 0;
}

uint8_t flash_shiftInMSB()
{
	unsigned char bitIndex;
	uint8_t val = 0;
	
	//FLASH_CLOCK = 1; // extra clock transition to get the first bit
	//FLASH_CLOCK = 0;
	
	// recv each bit of the byte (one clock period per iteration)
	for (bitIndex = 0; bitIndex < 8; ++bitIndex)
	{
		FLASH_CLOCK = 1;
		val <<= 1;
		val.0 = FLASH_DATA_IN;
		FLASH_CLOCK = 0;
	}
	
	FLASH_CLOCK = 0;

	return val;
}

// shift out 8 bits of data, LSB first.  leaves clock low.
void flash_shiftOutLSB(uint8_t val)
{
	unsigned char bitIndex;
	
	// send each bit of the byte (one clock period per iteration)
	for (bitIndex = 0; bitIndex < 8; ++bitIndex)
	{
		FLASH_CLOCK = 0;
		FLASH_DATA_OUT = val.0;
		val >>= 1;
		FLASH_CLOCK = 1;
	}
	
	FLASH_CLOCK = 0;
}

uint8_t flash_shiftInLSB()
{
	unsigned char bitIndex;
	uint8_t val = 0;
	
	// recv each bit of the byte (one clock period per iteration)
	for (bitIndex = 0; bitIndex < 8; ++bitIndex)
	{
		FLASH_CLOCK = 0;
		val >>= 1;
		val.7 = FLASH_DATA_IN;
		FLASH_CLOCK = 1;
	}
	
	FLASH_CLOCK = 0;

	return val;
}

void flash_eraseAll()
{
	flash_beginInstruction(WRITE_ENABLE);
	flash_endInstruction();
	flash_beginInstruction(BULK_ERASE);
	flash_endInstruction();
	delay_s(20); // 20sec is worst-case bulk erase time
}

void deleteSector(unsigned short sector)
{
}

void flash_prepPageForWrite(unsigned short page)
{
	flash_beginInstruction(WRITE_ENABLE);
	flash_endInstruction();
	flash_beginInstruction(PAGE_PROGRAM);
	
	flash_shiftOutMSB(page >> 8); // top few bits of page
	flash_shiftOutMSB(page & 0xFF); // bottom bits of page
	flash_shiftOutMSB(0); // send byte-on-page address as zero
}

void flash_endPageWrite()
{
	flash_endInstruction();
	delay_ms(5); // worst case
}

void flash_prepPageForRead(unsigned short page)
{
	flash_beginInstruction(READ);

	flash_shiftOutMSB(page >> 8); // top few bits of page
	flash_shiftOutMSB(page & 0xFF); // bottom bits of page
	flash_shiftOutMSB(0); // send byte-on-page address as zero	
}

void flash_endPageRead()
{
	flash_endInstruction();
}

void readPage(unsigned short page)
{
}

bool already_done = false;

unsigned char flash_test()
{
	already_done = true;
	if (!already_done)
	{
		//flash_eraseAll();
	
		flash_prepPageForWrite(0);
		flash_shiftOutLSB(47);
		flash_endPageWrite();
		
		flash_prepPageForWrite(12);
		flash_shiftOutLSB(69);
		flash_endPageWrite();
		already_done = true;
	}
	
	flash_prepPageForRead(0);
	unsigned char blah = flash_shiftInLSB();
	flash_endPageRead();
	
	flash_prepPageForRead(12);
	ext1 = flash_shiftInLSB();
	flash_endPageRead();
	
	return blah;
	
	/*if (blah == 0x30)
		return 1;
	else
		return 0;*/
}

void flash_readRecord(struct Record *rec)
{
	uint8_t * data = (uint8_t *)rec;
	
	unsigned char byteIndex;
	
	for (byteIndex = 0; byteIndex < sizeof(struct Record); ++byteIndex)
	{
		data[byteIndex] = flash_shiftInLSB();
	}
}